@@ -37,6 +37,9 @@ struct _GcalEventWidget
 
   GcalEvent     *event;
 
+  /* Drag n' Drop icon */
+  GdkPixbuf     *dnd_pixbuf;
+
   GdkWindow     *event_window;
   gboolean       button_pressed;
 };
@@ -103,7 +106,7 @@ update_color (GcalEventWidget *self)
   gchar *color_str;
   gchar *css_class;
   GDateTime *now;
-  gboolean date_compare;
+  gint date_compare;
 
   context = gtk_widget_get_style_context (GTK_WIDGET (self));
   color = gcal_event_get_color (self->event);
@@ -184,15 +187,26 @@ gcal_event_widget_set_event_internal (GcalEventWidget *self,
 static void
 gcal_event_widget_init (GcalEventWidget *self)
 {
+  GtkWidget *widget;
   GSettings *settings;
   gchar *clock_format;
 
+  widget = GTK_WIDGET (self);
   settings = g_settings_new ("org.gnome.desktop.interface");
   clock_format = g_settings_get_string (settings, "clock-format");
   self->clock_format_24h = g_strcmp0 (clock_format, "24h") == 0;
 
-  gtk_widget_set_has_window (GTK_WIDGET (self), FALSE);
-  gtk_widget_set_can_focus (GTK_WIDGET (self), TRUE);
+  gtk_widget_set_has_window (widget, FALSE);
+  gtk_widget_set_can_focus (widget, TRUE);
+
+  /* Setup the event widget as a drag source */
+  gtk_drag_source_set (widget,
+                       GDK_BUTTON1_MASK,
+                       NULL,
+                       0,
+                       GDK_ACTION_COPY);
+
+  gtk_drag_source_add_text_targets (widget);
 
   g_clear_object (&settings);
   g_free (clock_format);
@@ -265,6 +279,7 @@ gcal_event_widget_finalize (GObject *object)
 
   /* releasing properties */
   g_clear_pointer (&self->css_class, g_free);
+  g_clear_object (&self->dnd_pixbuf);
   g_clear_object (&self->event);
 
   G_OBJECT_CLASS (gcal_event_widget_parent_class)->finalize (object);
@@ -562,6 +577,15 @@ gcal_event_widget_draw (GtkWidget *widget,
       cairo_paint (cr);
     }
 
+  /* Setup the drag n' drop icon */
+  g_clear_object (&self->dnd_pixbuf);
+
+  self->dnd_pixbuf = gdk_pixbuf_get_from_window (self->event_window,
+                                                 0,
+                                                 0,
+                                                 width,
+                                                 height);
+
   pango_font_description_free (font_desc);
   g_object_unref (layout);
   g_free (display_text);
@@ -599,6 +623,14 @@ gcal_event_widget_button_release_event (GtkWidget      *widget,
   return FALSE;
 }
 
+static void
+gcal_event_widget_drag_begin (GtkWidget      *widget,
+                              GdkDragContext *context)
+{
+  GcalEventWidget *self = GCAL_EVENT_WIDGET (widget);
+
+  gtk_drag_set_icon_pixbuf (context, self->dnd_pixbuf, 0, 0);
+}
 
 static void
 gcal_event_widget_class_init(GcalEventWidgetClass *klass)
@@ -622,6 +654,7 @@ gcal_event_widget_class_init(GcalEventWidgetClass *klass)
   widget_class->draw = gcal_event_widget_draw;
   widget_class->button_press_event = gcal_event_widget_button_press_event;
   widget_class->button_release_event = gcal_event_widget_button_release_event;
+  widget_class->drag_begin = gcal_event_widget_drag_begin;
 
   /**
    * GcalEventWidget::date-end:
