@@ -113,7 +113,6 @@ struct _GcalWindow
   GtkWidget           *source_dialog;
   gint                 refresh_timeout;
   gint                 refresh_timeout_id;
-  gint                 open_edit_dialog_timeout_id;
 
   /* temp to keep event_creation */
   gboolean             open_edit_dialog;
@@ -138,6 +137,10 @@ enum
 }
 
 
+static void           on_show_calendars_action_activated (GSimpleAction       *action,
+                                                          GVariant            *param,
+                                                          gpointer             user_data);
+
 static void           on_date_action_activated           (GSimpleAction       *action,
                                                           GVariant            *param,
                                                           gpointer             user_data);
@@ -205,17 +208,10 @@ static void           remove_source                      (GcalManager         *m
                                                           ESource             *source,
                                                           gpointer             user_data);
 
-static void           show_source_dialog                 (GtkButton           *button,
-                                                          gpointer             user_data);
-
 static void           source_row_activated               (GtkListBox          *listbox,
                                                           GtkListBoxRow       *row,
                                                           gpointer             user_data);
 
-static void           source_changed                     (GcalWindow          *window,
-                                                          ESource             *source);
-
-
 static void           on_calendar_toggled                (GObject             *object,
                                                           GParamSpec          *pspec,
                                                           gpointer             user_data);
@@ -285,8 +281,23 @@ static const GActionEntry actions[] = {
   {"previous", on_date_action_activated },
   {"today",    on_date_action_activated },
   {"change-view", on_view_action_activated, "i" },
+  {"show-calendars", on_show_calendars_action_activated },
 };
 
+static void
+on_show_calendars_action_activated (GSimpleAction *action,
+                                    GVariant      *param,
+                                    gpointer       user_data)
+{
+  GcalWindow *window = GCAL_WINDOW (user_data);
+
+  gcal_source_dialog_set_mode (GCAL_SOURCE_DIALOG (window->source_dialog), GCAL_SOURCE_DIALOG_MODE_NORMAL);
+
+  gtk_widget_hide (window->calendar_popover);
+
+  gtk_widget_show (window->source_dialog);
+}
+
 static void
 on_date_action_activated (GSimpleAction *action,
                           GVariant      *param,
@@ -800,7 +811,6 @@ add_source (GcalManager *manager,
   window = GCAL_WINDOW (user_data);
 
   row = make_row_from_source (GCAL_WINDOW (user_data), source);
-  g_object_set_data (G_OBJECT (row), "source", source);
 
   gtk_container_add (GTK_CONTAINER (window->calendar_listbox), row);
 }
@@ -858,6 +868,9 @@ make_row_from_source (GcalWindow *window,
   gtk_container_add (GTK_CONTAINER (box), checkbox);
   gtk_container_add (GTK_CONTAINER (row), box);
 
+  g_object_set_data (G_OBJECT (row), "check", checkbox);
+  g_object_set_data (G_OBJECT (row), "source", source);
+
   gtk_widget_show_all (row);
 
   g_object_unref (pixbuf);
@@ -889,19 +902,6 @@ remove_source (GcalManager *manager,
   g_list_free (children);
 }
 
-static void
-show_source_dialog (GtkButton *button,
-                    gpointer   user_data)
-{
-  GcalWindow *window = GCAL_WINDOW (user_data);
-
-  gcal_source_dialog_set_mode (GCAL_SOURCE_DIALOG (window->source_dialog), GCAL_SOURCE_DIALOG_MODE_NORMAL);
-
-  gtk_widget_hide (window->calendar_popover);
-
-  gtk_widget_show (window->source_dialog);
-}
-
 static void
 source_row_activated (GtkListBox    *listbox,
                       GtkListBoxRow *row,
@@ -924,9 +924,37 @@ source_row_activated (GtkListBox    *listbox,
   gtk_widget_show (window->source_dialog);
 }
 
+static void
+source_enabled (GcalWindow *self,
+                ESource    *source,
+                gboolean    enabled)
+{
+  GList *children, *aux;
+
+  children = gtk_container_get_children (GTK_CONTAINER (self->calendar_listbox));
+
+  for (aux = children; aux != NULL; aux = aux->next)
+    {
+      ESource *child_source = g_object_get_data (G_OBJECT (aux->data), "source");
+
+      if (child_source != NULL && child_source == source)
+        {
+          GtkToggleButton *button = g_object_get_data (G_OBJECT (aux->data), "check");
+
+          g_signal_handlers_block_by_func (button, on_calendar_toggled, self);
+          gtk_toggle_button_set_active (button, enabled);
+          g_signal_handlers_unblock_by_func (button, on_calendar_toggled, self);
+
+          break;
+        }
+    }
+
+  g_list_free (children);
+}
+
 static void
 source_changed (GcalWindow *window,
-                ESource     *source)
+                ESource    *source)
 {
   GList *children, *aux;
 
@@ -1204,17 +1232,16 @@ schedule_open_edit_dialog_by_uuid (OpenEditDialogData *edit_dialog_data)
   widgets = gcal_view_get_children_by_uuid (GCAL_VIEW (window->month_view), edit_dialog_data->uuid);
   if (widgets != NULL)
     {
-      window->open_edit_dialog_timeout_id = 0;
-
       event_activated (NULL, widgets->data, edit_dialog_data->window);
       g_list_free (widgets);
       g_free (edit_dialog_data->uuid);
       g_free (edit_dialog_data);
-
-      return G_SOURCE_REMOVE;
+      return FALSE;
+    }
+  else
+    {
+      return TRUE;
     }
-
-  return G_SOURCE_CONTINUE;
 }
 
 static void
@@ -1300,7 +1327,6 @@ gcal_window_class_init(GcalWindowClass *klass)
   gtk_widget_class_bind_template_child (widget_class, GcalWindow, notification_action_button);
   gtk_widget_class_bind_template_child (widget_class, GcalWindow, notification_close_button);
 
-  gtk_widget_class_bind_template_callback (widget_class, show_source_dialog);
   gtk_widget_class_bind_template_callback (widget_class, source_row_activated);
 
   gtk_widget_class_bind_template_callback (widget_class, key_pressed);
@@ -1434,12 +1460,6 @@ gcal_window_finalize (GObject *object)
       window->save_geometry_timeout_id = 0;
     }
 
-  if (window->open_edit_dialog_timeout_id > 0)
-    {
-      g_source_remove (window->open_edit_dialog_timeout_id);
-      window->open_edit_dialog_timeout_id = 0;
-    }
-
   /* If we have a queued event to delete, remove it now */
   if (window->event_to_delete)
     {
@@ -1483,10 +1503,11 @@ gcal_window_set_property (GObject      *object,
         {
           g_signal_connect (self->manager, "source-added", G_CALLBACK (add_source), object);
           g_signal_connect (self->manager, "source-removed", G_CALLBACK (remove_source), object);
-          g_signal_connect_swapped (self->manager, "source-enabled", G_CALLBACK (source_changed), object);
+          g_signal_connect_swapped (self->manager, "source-enabled", G_CALLBACK (source_enabled), object);
           g_signal_connect_swapped (self->manager, "source-changed", G_CALLBACK (source_changed), object);
 
           gcal_edit_dialog_set_manager (GCAL_EDIT_DIALOG (self->edit_dialog), self->manager);
+          gcal_month_view_set_manager (GCAL_MONTH_VIEW (self->month_view), self->manager);
           gcal_year_view_set_manager (GCAL_YEAR_VIEW (self->year_view), self->manager);
           gcal_quick_add_popover_set_manager (GCAL_QUICK_ADD_POPOVER (self->quick_add_popover), self->manager);
           gcal_source_dialog_set_manager (GCAL_SOURCE_DIALOG (self->source_dialog), self->manager);
@@ -1682,8 +1703,6 @@ gcal_window_open_event_by_uuid (GcalWindow  *window,
       OpenEditDialogData *edit_dialog_data = g_new0 (OpenEditDialogData, 1);
       edit_dialog_data->window = window;
       edit_dialog_data->uuid = g_strdup (uuid);
-      window->open_edit_dialog_timeout_id = g_timeout_add_seconds (2,
-                                                                   (GSourceFunc) schedule_open_edit_dialog_by_uuid,
-                                                                   edit_dialog_data);
+      g_timeout_add_seconds (2, (GSourceFunc) schedule_open_edit_dialog_by_uuid, edit_dialog_data);
     }
 }
