@@ -25,9 +25,7 @@
 #include <math.h>
 #include <string.h>
 
-#define NAVIGATOR_CELL_WIDTH (210 + 15)
-#define NAVIGATOR_CELL_HEIGHT 210
-#define SIDEBAR_PREFERRED_WIDTH 200
+#define NAVIGATOR_MAX_GRID_SIZE 4
 #define VISUAL_CLUES_SIDE 3.0
 #define WEEK_NUMBER_MARGIN 3.0
 
@@ -36,6 +34,8 @@ typedef struct
   /* month span from 0 to 11 */
   gint start_day, start_month;
   gint end_day, end_month;
+  gint hovered_day, hovered_month;
+  gint dnd_day, dnd_month;
 } ButtonData;
 
 typedef struct
@@ -67,6 +67,11 @@ struct _GcalYearView
 
   /* geometry info */
   GridData     *navigator_grid;
+  guint         number_of_columns;
+  guint         column_width;
+  guint         row_height;
+  guint         header_height;
+  guint         sidebar_width;
 
   /* state flags */
   gboolean      popover_mode;
@@ -402,6 +407,8 @@ update_sidebar (GcalYearView *year_view)
   for (l = events; l != NULL; l = g_list_next (l))
     add_event_to_day_array (year_view, l->data, days_widgets_array, days_span);
 
+  gtk_widget_queue_draw (year_view->navigator);
+
   for (i = 0; i < days_span; i++)
     {
       GList *current_day = days_widgets_array[i];
@@ -540,33 +547,41 @@ sidebar_sort_func (GtkListBoxRow *row1,
   return result;
 }
 
+
 static gboolean
 calculate_day_month_for_coord (GcalYearView *year_view,
                                gdouble       x,
                                gdouble       y,
                                gint         *out_day,
-                               gint         *out_month)
+                               gint         *out_month,
+                               gboolean     *is_title)
 {
-  gint row, column, i, sw, clicked_cell, day, month;
+  gint row, column, i, sw, clicked_cell, day, month, columns_or_rows;
+  gint number_of_rows;
   gdouble box_side;
 
   row = -1;
   column = -1;
   box_side = year_view->navigator_grid->box_side;
+  number_of_rows = ceil (12.0 / year_view->number_of_columns);
   sw = 1 - 2 * year_view->k;
 
-  /* y selection */
-  for (i = 0; i < 4 && ((row == -1) || (column == -1)); i++)
+  *is_title = FALSE;
+  columns_or_rows = year_view->number_of_columns > number_of_rows ? year_view->number_of_columns : number_of_rows;
+
+  for (i = 0; i < columns_or_rows && ((row == -1) || (column == -1)); i++)
     {
       if (row == -1 &&
-          y > year_view->navigator_grid->coordinates[i * 4].y &&
-          y < year_view->navigator_grid->coordinates[i * 4].y + box_side * 7)
+          y > year_view->navigator_grid->coordinates[i * year_view->number_of_columns].y &&
+          y < year_view->navigator_grid->coordinates[i * year_view->number_of_columns].y + year_view->row_height)
         {
+          if (y < year_view->navigator_grid->coordinates[i * year_view->number_of_columns].y + box_side)
+            *is_title = TRUE;
           row = i;
         }
       if (column == -1 &&
-          x > year_view->navigator_grid->coordinates[i].x + box_side * (1 - year_view->k) &&
-          x < year_view->navigator_grid->coordinates[i].x + box_side * (8 - year_view->k))
+          x > year_view->navigator_grid->coordinates[i].x + box_side * year_view->show_week_numbers * (1 - year_view->k) &&
+          x < year_view->navigator_grid->coordinates[i].x + year_view->column_width - box_side * year_view->show_week_numbers * year_view->k)
         {
           column = i;
         }
@@ -575,9 +590,18 @@ calculate_day_month_for_coord (GcalYearView *year_view,
   if (row == -1 || column == -1)
     return FALSE;
 
-  month = row * 4 + column;
+  month = row * year_view->number_of_columns + column;
+
+  if (month < 0 || month > 11)
+    return FALSE;
+
+  *out_month = month;
+
+  if (*is_title)
+    return TRUE;
+
   row = (y - (year_view->navigator_grid->coordinates[month].y + box_side)) / box_side;
-  column = (x - (year_view->navigator_grid->coordinates[month].x + box_side * (1 - year_view->k))) / box_side;
+  column = (x - (year_view->navigator_grid->coordinates[month].x + box_side * year_view->show_week_numbers * (1 - year_view->k))) / box_side;
   clicked_cell = row * 7 + column;
   day = 7 * ((clicked_cell + 7 * year_view->k) / 7) + sw * (clicked_cell % 7) + (1 - year_view->k);
   day -= ((time_day_of_week (1, month, year_view->date->year) - year_view->first_weekday + 7) % 7);
@@ -586,7 +610,6 @@ calculate_day_month_for_coord (GcalYearView *year_view,
     return FALSE;
 
   *out_day = day;
-  *out_month = month;
   return TRUE;
 }
 
@@ -640,11 +663,11 @@ draw_month_grid (GcalYearView *year_view,
   layout = gtk_widget_create_pango_layout (widget, str);
   pango_layout_set_font_description (layout, font_desc);
   pango_layout_get_pixel_size (layout, &layout_width, &layout_height);
-  gtk_render_layout (context, cr, x + (box_side * 8 - layout_width) / 2.0, y + (box_side - layout_height) / 2.0,
+  gtk_render_layout (context, cr, x + (year_view->column_width - layout_width) / 2.0, y + (box_side - layout_height) / 2.0,
                      layout);
 
   gtk_render_background (context, cr,
-                         x + (box_side * 8 - layout_width) / 2.0, y + (box_side - layout_height) / 2.0,
+                         x + (year_view->column_width - layout_width) / 2.0, y + (box_side - layout_height) / 2.0,
                          layout_width, layout_width);
 
   pango_font_description_free (font_desc);
@@ -658,7 +681,8 @@ draw_month_grid (GcalYearView *year_view,
   gtk_style_context_get_color (context, state_flags, &color);
   cairo_set_line_width (cr, 0.2);
   gdk_cairo_set_source_rgba (cr, &color);
-  cairo_move_to (cr, x + box_side * (1 - year_view->k), y + box_side + 0.4);
+
+  cairo_move_to (cr, x + box_side * year_view->show_week_numbers * (1 - year_view->k), y + box_side + 0.4);
   cairo_rel_line_to (cr, 7.0 * box_side, 0);
   cairo_stroke (cr);
 
@@ -674,7 +698,7 @@ draw_month_grid (GcalYearView *year_view,
   days_delay = (time_day_of_week (1, month_nr, year_view->date->year) - year_view->first_weekday + 7) % 7;
   days = days_delay + icaltime_days_in_month (month_nr + 1, year_view->date->year);
   shown_rows = ceil (days / 7.0);
-  sunday_idx = year_view->k * 6 + sw * ((7 - year_view->first_weekday) % 7) + 1;
+  sunday_idx = year_view->k * 7 + sw * ((7 - year_view->first_weekday) % 7) + year_view->show_week_numbers * (1 - year_view->k);
 
   start_date.day    = 1;
   start_date.month  = month_nr + 1;
@@ -691,7 +715,7 @@ draw_month_grid (GcalYearView *year_view,
 
   for (i = 0; i < 7 * shown_rows; i++)
     {
-      column = (i % 7) + 1;
+      column = (i % 7) + (year_view->k || year_view->show_week_numbers);
       row = i / 7;
 
       j = 7 * ((i + 7 * year_view->k) / 7) + sw * (i % 7) + (1 - year_view->k);
@@ -707,6 +731,49 @@ draw_month_grid (GcalYearView *year_view,
       box_padding_top = (box_side - layout_height) / 2 > 0 ? (box_side - layout_height) / 2 : 0;
       box_padding_start = (box_side - layout_width) / 2 > 0 ? (box_side - layout_width) / 2 : 0;
 
+      /* Draw the hover background */
+      if (year_view->selected_data->hovered_day == j &&
+          year_view->selected_data->hovered_month == month_nr)
+        {
+          gint hover_x, hover_y;
+
+          hover_x = box_side * column + x + box_padding_start / 2.0 - year_view->k * box_side - 2.0;
+          hover_y = box_side * (row + 1) + y + box_padding_top - 1.0;
+
+          gtk_style_context_set_state (context, state_flags | GTK_STATE_FLAG_PRELIGHT);
+
+          gtk_render_background (context,
+                                 cr,
+                                 hover_x,
+                                 hover_y,
+                                 box_side - box_padding_start / 2.0,
+                                 box_side);
+
+          gtk_style_context_set_state (context, state_flags);
+        }
+
+      /* Draw the Drag n' Drop indicator */
+      if (year_view->selected_data->dnd_day == j &&
+          year_view->selected_data->dnd_month == month_nr)
+        {
+          gint dnd_x, dnd_y;
+
+          dnd_x = box_side * column + x + box_padding_start / 2.0 - year_view->k * box_side - 2.0;
+          dnd_y = box_side * (row + 1) + y + box_padding_top - 1.0;
+
+          gtk_style_context_save (context);
+          gtk_style_context_add_class (context, "dnd");
+
+          gtk_render_background (context,
+                                 cr,
+                                 dnd_x,
+                                 dnd_y,
+                                 box_side - box_padding_start / 2.0,
+                                 box_side);
+
+          gtk_style_context_restore (context);
+        }
+
       selected_day = FALSE;
       if (year_view->selected_data->start_day != 0)
         {
@@ -882,8 +949,8 @@ draw_navigator (GcalYearView *year_view,
   GtkStateFlags state_flags;
 
   gint header_padding_left, header_padding_top, header_height, layout_width, layout_height;
-  gint real_padding_left, real_padding_top, i, sw, weeks_counter;
-  gdouble width, height, box_side;
+  gint real_padding_left, real_padding_top, i, sw, weeks_counter, number_of_rows;
+  gdouble width, height;
 
   gchar *header_str;
 
@@ -894,6 +961,7 @@ draw_navigator (GcalYearView *year_view,
   state_flags = gtk_style_context_get_state (context);
   sw = 1 - 2 * year_view->k;
   width = gtk_widget_get_allocated_width (widget);
+  number_of_rows = ceil (12.0 / year_view->number_of_columns);
 
   /* read header from CSS code related to the view */
   gtk_style_context_save (context);
@@ -925,23 +993,20 @@ draw_navigator (GcalYearView *year_view,
   header_height = header_padding_top * 2 + layout_height;
   height = gtk_widget_get_allocated_height (widget) - header_height;
 
-  if (((width / 4.0) / 8.0) < ((height / 3.0) / 7.0))
-    box_side = (width / 4.0) / 8.0;
-  else
-    box_side = (height / 3.0) / 7.0;
+  real_padding_left = (width - year_view->column_width * year_view->number_of_columns) / (year_view->number_of_columns + 1);
+  real_padding_top = (height - year_view->row_height * number_of_rows) / number_of_rows;
 
-  real_padding_left = (width - (8 * 4 * box_side)) / 5.0;
-  real_padding_top = (height - (7 * 3 * box_side)) / 4.0;
+  if (real_padding_top < 0)
+    real_padding_top = 0;
 
-  year_view->navigator_grid->box_side = box_side;
   weeks_counter = year_view->first_week_of_year;
   for (i = 0; i < 12; i++)
     {
-      gint row = i / 4;
-      gint column = year_view->k * 3 + sw * (i % 4);
+      gint row = i / year_view->number_of_columns;
+      gint column = year_view->k * (year_view->number_of_columns - 1) + sw * (i % year_view->number_of_columns);
 
-      year_view->navigator_grid->coordinates[i].x = (column + 1) * real_padding_left + column * box_side * 8;
-      year_view->navigator_grid->coordinates[i].y = (row + 1) * real_padding_top + row * box_side * 7 + header_height;
+      year_view->navigator_grid->coordinates[i].x = (column + 1) * real_padding_left + column * year_view->column_width;
+      year_view->navigator_grid->coordinates[i].y = row * real_padding_top + row * year_view->row_height + header_height;
       draw_month_grid (year_view, widget, cr, i, &weeks_counter);
     }
 
@@ -954,9 +1019,14 @@ navigator_button_press_cb (GcalYearView   *year_view,
                            GtkWidget      *widget)
 {
   gint day, month;
-  if (!calculate_day_month_for_coord (year_view, event->x, event->y, &day, &month))
+  gboolean is_title = FALSE;
+
+  if (!calculate_day_month_for_coord (year_view, event->x, event->y, &day, &month, &is_title))
     return FALSE;
 
+  if (is_title)
+    day = 1;
+
   year_view->button_pressed = TRUE;
   year_view->selected_data->start_day = day;
   year_view->selected_data->start_month = month;
@@ -970,13 +1040,17 @@ navigator_button_release_cb (GcalYearView   *year_view,
                              GtkWidget      *widget)
 {
   gint day, month;
+  gboolean is_title = FALSE;
 
   if (!year_view->button_pressed)
     return FALSE;
 
-  if (!calculate_day_month_for_coord (year_view, event->x, event->y, &day, &month))
+  if (!calculate_day_month_for_coord (year_view, event->x, event->y, &day, &month, &is_title))
     goto fail;
 
+  if (is_title)
+    day = g_date_get_days_in_month (month + 1, year_view->date->year);
+
   year_view->button_pressed = FALSE;
   year_view->selected_data->end_day = day;
   year_view->selected_data->end_month = month;
@@ -1023,23 +1097,39 @@ navigator_motion_notify_cb (GcalYearView   *year_view,
                             GtkWidget      *widget)
 {
   gint day, month;
+  gboolean is_title = FALSE;
 
-  if (!year_view->button_pressed)
-    return FALSE;
+  /* Cancel the hover when selecting a date range */
+  year_view->selected_data->hovered_day = -1;
+  year_view->selected_data->hovered_month = -1;
 
-  if (!calculate_day_month_for_coord (year_view, event->x, event->y, &day, &month))
-    goto fail;
+  if (calculate_day_month_for_coord (year_view, event->x, event->y, &day, &month, &is_title))
+    {
+      if (year_view->button_pressed)
+        {
+          if (is_title)
+            day = g_date_get_days_in_month (month + 1, year_view->date->year);
+
+          year_view->selected_data->end_day = day;
+          year_view->selected_data->end_month = month;
+          gtk_widget_queue_draw (widget);
+
+          return GDK_EVENT_STOP;
+        }
+      else
+        {
+          /* We only deal with the hover effect when the button is not pressed */
+          year_view->selected_data->hovered_day = day;
+          year_view->selected_data->hovered_month = month;
+          gtk_widget_queue_draw (widget);
+
+          return GDK_EVENT_PROPAGATE;
+        }
+    }
 
-  year_view->selected_data->end_day = day;
-  year_view->selected_data->end_month = month;
   gtk_widget_queue_draw (widget);
-  return TRUE;
 
-fail:
-  year_view->selected_data->end_day = year_view->selected_data->start_day;
-  year_view->selected_data->end_month = year_view->selected_data->start_month;
-  gtk_widget_queue_draw (widget);
-  return TRUE;
+  return GDK_EVENT_STOP;
 }
 
 static void
@@ -1086,6 +1176,218 @@ popover_closed_cb (GcalYearView *year_view,
   reset_sidebar (year_view);
 }
 
+static void
+calculate_grid_sizes (GcalYearView *self)
+{
+  /* 1 column for week number and 7 for days */
+  if (self->show_week_numbers)
+    self->column_width = self->navigator_grid->box_side * 8;
+  else
+    self->column_width = self->navigator_grid->box_side * 7;
+
+  /* 1 line for month header and 6 lines for weeks */
+  self->row_height = self->navigator_grid->box_side * 7;
+}
+
+static void
+calculate_sizes (GcalYearView *self)
+{
+  GtkStyleContext *context;
+  PangoLayout *layout;
+  PangoFontDescription *font;
+
+  gint padding_left, padding_right, padding_top, padding_bottom;
+  gint width, height, box_width, box_height, natural;
+  gchar *test_str;
+
+  /* get header info from CSS */
+  context = gtk_widget_get_style_context (GTK_WIDGET (self));
+  gtk_style_context_save (context);
+  gtk_style_context_add_class (context, "first-view-header");
+  gtk_style_context_get (context, gtk_style_context_get_state (context),
+                         "padding-top", &padding_top,
+                         "padding-bottom", &padding_bottom,
+                         "font", &font, NULL);
+  gtk_style_context_restore (context);
+
+  /* measure header height */
+  test_str = g_strdup_printf ("8888");
+  layout = gtk_widget_create_pango_layout (self->navigator, test_str);
+  pango_layout_set_font_description (layout, font);
+  pango_layout_get_pixel_size (layout, &width, &height);
+
+  pango_font_description_free (font);
+  g_object_unref (layout);
+  g_free (test_str);
+
+  self->header_height = padding_top + height + padding_bottom;
+
+  /* get day info from CSS */
+  context = gtk_widget_get_style_context (GTK_WIDGET (self->navigator));
+  gtk_style_context_save (context);
+  gtk_style_context_add_class (context, "days");
+  gtk_style_context_get (context, gtk_style_context_get_state (context),
+                         "padding-left", &padding_left,
+                         "padding-right", &padding_right,
+                         "padding-top", &padding_top,
+                         "padding-bottom", &padding_bottom,
+                         "font", &font, NULL);
+  gtk_style_context_restore (context);
+
+  /* measure box size */
+  test_str = g_strdup_printf ("88");
+  layout = gtk_widget_create_pango_layout (self->navigator, test_str);
+  pango_layout_set_font_description (layout, font);
+  pango_layout_get_pixel_size (layout, &width, &height);
+
+  pango_font_description_free (font);
+  g_object_unref (layout);
+  g_free (test_str);
+
+  box_width = padding_left + width + padding_right;
+  box_height = padding_top + height + padding_bottom;
+
+  self->navigator_grid->box_side = (box_width > box_height) ? box_width : box_height;
+
+  /* get sidebar width */
+  gtk_widget_get_preferred_width (self->sidebar, &width, &natural);
+  self->sidebar_width = width;
+
+  calculate_grid_sizes (self);
+}
+
+/*
+ * Drag and Drop functions
+ */
+static gboolean
+navigator_drag_motion_cb (GcalYearView   *self,
+                          GdkDragContext *context,
+                          gint            x,
+                          gint            y,
+                          guint           time,
+                          GtkWidget      *navigator)
+{
+  gint day, month;
+  gboolean is_title, retval;
+
+  retval = FALSE;
+
+  self->selected_data->dnd_day = -1;
+  self->selected_data->dnd_month = -1;
+
+  if (calculate_day_month_for_coord (self, x, y, &day, &month, &is_title))
+    {
+      /* For now, don't allow dropping events on the month title */
+      if (!is_title)
+        {
+          self->selected_data->dnd_day = day;
+          self->selected_data->dnd_month = month;
+
+          gdk_drag_status (context, GDK_ACTION_COPY, time);
+
+          retval = TRUE;
+        }
+    }
+  else
+    {
+      gdk_drag_status (context, 0, time);
+    }
+
+  gtk_widget_queue_draw (self->navigator);
+
+  return retval;
+}
+
+static gboolean
+navigator_drag_drop_cb (GcalYearView   *self,
+                        GdkDragContext *context,
+                        gint            x,
+                        gint            y,
+                        guint           time,
+                        GtkWidget      *navigator)
+{
+  gint day, month, diff;
+  gboolean is_title;
+
+  if (calculate_day_month_for_coord (self, x, y, &day, &month, &is_title))
+    {
+      if (!is_title)
+        {
+          GcalEventWidget *event_widget;
+          GcalEvent *event;
+          GDateTime *start_dt, *end_dt;
+          GDateTime *drop_date;
+
+          event_widget = GCAL_EVENT_WIDGET (gtk_drag_get_source_widget (context));
+          event = gcal_event_widget_get_event (event_widget);
+          start_dt = gcal_event_get_date_start (event);
+          end_dt = gcal_event_get_date_end (event);
+
+          /*
+           * The difference is calculated in number of days. Here we have to
+           * use the start_dt's hour/minute/second since otherwise we may not
+           * calculate the difference correctly.
+           */
+          drop_date = g_date_time_new_local (self->date->year,
+                                             month + 1,
+                                             day,
+                                             g_date_time_get_hour (start_dt),
+                                             g_date_time_get_minute (start_dt),
+                                             g_date_time_get_second (start_dt));
+
+          diff = g_date_time_difference (drop_date, start_dt) / G_TIME_SPAN_DAY;
+
+          if (diff != 0)
+            {
+              GDateTime *new_start;
+
+              new_start = g_date_time_add_days (start_dt, diff);
+              gcal_event_set_date_start (event, new_start);
+
+              /* The event may have a NULL end date, so we have to check it here */
+              if (end_dt)
+                {
+                  GDateTime *new_end;
+
+                  new_end = g_date_time_add_days (end_dt, diff);
+                  gcal_event_set_date_end (event, new_start);
+
+                  g_clear_pointer (&new_end, g_date_time_unref);
+                }
+
+              gcal_manager_update_event (self->manager, event);
+
+              g_clear_pointer (&new_start, g_date_time_unref);
+            }
+
+          g_clear_pointer (&drop_date, g_date_time_unref);
+        }
+    }
+
+  /* Cancel the DnD after the event is dropped */
+  self->selected_data->dnd_day = -1;
+  self->selected_data->dnd_month = -1;
+
+  gtk_drag_finish (context, TRUE, FALSE, time);
+
+  gtk_widget_queue_draw (navigator);
+
+  return TRUE;
+}
+
+static void
+navigator_drag_leave_cb (GcalYearView   *self,
+                         GdkDragContext *context,
+                         guint           time,
+                         GtkWidget      *navigator)
+{
+  /* Cancel the DnD after the event is dropped */
+  self->selected_data->dnd_day = -1;
+  self->selected_data->dnd_month = -1;
+
+  gtk_widget_queue_draw (navigator);
+}
+
 static void
 gcal_year_view_finalize (GObject *object)
 {
@@ -1145,7 +1447,9 @@ gcal_year_view_set_property (GObject      *object,
       if (self->show_week_numbers != g_value_get_boolean (value))
         {
           self->show_week_numbers = g_value_get_boolean (value);
+          calculate_grid_sizes (self);
           g_object_notify (object, "show-week-numbers");
+          gtk_widget_queue_allocate (GTK_WIDGET (self));
         }
       break;
 
@@ -1161,40 +1465,20 @@ gcal_year_view_get_preferred_width (GtkWidget *widget,
 {
   GcalYearView *year_view = GCAL_YEAR_VIEW (widget);
   GtkStyleContext *context;
-  gint padding_left;
+  gint padding_left, padding_right, hpadding;
 
   context = gtk_widget_get_style_context (year_view->navigator);
 
   gtk_style_context_get (context,
                          gtk_style_context_get_state (context),
-                         "padding-left", &padding_left, NULL);
+                         "padding-left", &padding_left,
+                         "padding-right", &padding_right, NULL);
 
+  hpadding = padding_left + padding_right;
   if (minimum != NULL)
-    *minimum = NAVIGATOR_CELL_WIDTH * 4 + padding_left * 8;
+    *minimum = hpadding + year_view->column_width;
   if (natural != NULL)
-    *natural = NAVIGATOR_CELL_WIDTH * 4 + padding_left * 8 + SIDEBAR_PREFERRED_WIDTH;
-}
-
-static void
-gcal_year_view_get_preferred_height_for_width (GtkWidget *widget,
-                                               gint       width,
-                                               gint      *minimum,
-                                               gint      *natural)
-{
-  GcalYearView *year_view = GCAL_YEAR_VIEW (widget);
-  GtkStyleContext *context;
-  gint padding_top;
-
-  context = gtk_widget_get_style_context (year_view->navigator);
-
-  gtk_style_context_get (context,
-                         gtk_style_context_get_state (context),
-                         "padding-top", &padding_top, NULL);
-
-  if (minimum != NULL)
-    *minimum = NAVIGATOR_CELL_HEIGHT * 3 + padding_top * 6;
-  if (natural != NULL)
-    *natural = NAVIGATOR_CELL_HEIGHT * 3 + padding_top * 6;
+    *natural = hpadding + year_view->column_width + year_view->sidebar_width;
 }
 
 static void
@@ -1203,15 +1487,36 @@ gcal_year_view_size_allocate (GtkWidget     *widget,
 {
   GcalYearView *year_view = GCAL_YEAR_VIEW (widget);
   GtkStyleContext *context;
-  gint padding_left;
+  gint padding_left, padding_right, padding_top, padding_bottom, hpadding, vpadding;
+  gint number_of_rows;
 
   context = gtk_widget_get_style_context (widget);
   gtk_style_context_save (context);
   gtk_style_context_add_class (context, "year-navigator");
-  gtk_style_context_get (context, gtk_style_context_get_state (context), "padding-left", &padding_left, NULL);
+  gtk_style_context_get (context,
+                         gtk_style_context_get_state (context),
+                         "padding-left", &padding_left,
+                         "padding-right", &padding_right,
+                         "padding-top", &padding_top,
+                         "padding-bottom", &padding_bottom, NULL);
   gtk_style_context_restore (context);
 
-  year_view->popover_mode = (alloc->width < NAVIGATOR_CELL_WIDTH * 4 + padding_left * 8 + SIDEBAR_PREFERRED_WIDTH);
+  hpadding = padding_left + padding_right;
+  vpadding = padding_top + padding_bottom;
+
+  year_view->popover_mode = (alloc->width < (hpadding + year_view->column_width * 2 + year_view->sidebar_width));
+
+  year_view->number_of_columns = (alloc->width - hpadding - year_view->sidebar_width * !year_view->popover_mode) / year_view->column_width;
+
+  if (year_view->number_of_columns > NAVIGATOR_MAX_GRID_SIZE)
+    year_view->number_of_columns = NAVIGATOR_MAX_GRID_SIZE;
+
+  number_of_rows = ceil (12.0 / year_view->number_of_columns);
+
+  gtk_widget_set_size_request (year_view->navigator,
+                               hpadding + year_view->column_width * year_view->number_of_columns,
+                               vpadding + year_view->row_height * number_of_rows + year_view->header_height);
+
   if (year_view->popover_mode && !gtk_widget_is_ancestor (year_view->events_sidebar, year_view->popover))
     {
       g_object_ref (year_view->sidebar);
@@ -1343,6 +1648,8 @@ gcal_year_view_component_added (ECalDataModelSubscriber *subscriber,
       g_list_free (current_day);
     }
 
+  gtk_widget_queue_draw (year_view->navigator);
+
 out:
   g_clear_object (&event);
   g_free (days_widgets_array);
@@ -1443,7 +1750,6 @@ gcal_year_view_class_init (GcalYearViewClass *klass)
   object_class->set_property = gcal_year_view_set_property;
 
   widget_class->get_preferred_width = gcal_year_view_get_preferred_width;
-  widget_class->get_preferred_height_for_width = gcal_year_view_get_preferred_height_for_width;
   widget_class->size_allocate = gcal_year_view_size_allocate;
   widget_class->direction_changed = gcal_year_view_direction_changed;
 
@@ -1476,6 +1782,9 @@ gcal_year_view_class_init (GcalYearViewClass *klass)
   gtk_widget_class_bind_template_callback (widget_class, draw_navigator);
   gtk_widget_class_bind_template_callback (widget_class, navigator_button_press_cb);
   gtk_widget_class_bind_template_callback (widget_class, navigator_button_release_cb);
+  gtk_widget_class_bind_template_callback (widget_class, navigator_drag_drop_cb);
+  gtk_widget_class_bind_template_callback (widget_class, navigator_drag_leave_cb);
+  gtk_widget_class_bind_template_callback (widget_class, navigator_drag_motion_cb);
   gtk_widget_class_bind_template_callback (widget_class, navigator_motion_notify_cb);
   gtk_widget_class_bind_template_callback (widget_class, add_event_clicked_cb);
   gtk_widget_class_bind_template_callback (widget_class, popover_closed_cb);
@@ -1506,8 +1815,19 @@ gcal_year_view_init (GcalYearView *self)
   g_settings_bind (self->shell_settings, "show-weekdate", self, "show-week-numbers", G_SETTINGS_BIND_DEFAULT);
   g_signal_connect_swapped (self->shell_settings, "changed::show-weekdate", G_CALLBACK (gtk_widget_queue_draw), self);
 
+  /* layout */
+  self->number_of_columns = 4;
+  calculate_sizes (self);
+
   gtk_list_box_set_header_func (GTK_LIST_BOX (self->events_sidebar), update_sidebar_headers, self, NULL);
   gtk_list_box_set_sort_func (GTK_LIST_BOX (self->events_sidebar), sidebar_sort_func, NULL, NULL);
+
+  /* The year navigator is a DnD destination */
+  gtk_drag_dest_set (self->navigator,
+                     0,
+                     NULL,
+                     0,
+                     GDK_ACTION_COPY);
 }
 
 static void
